package net.sourceforge.nrl.parser.model;

import java.io.File;

import junit.framework.TestCase;
import net.sourceforge.nrl.parser.model.xsd.XSDModelLoader;

import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.xsd.util.XSDResourceFactoryImpl;

public class ModelUtilsTest extends TestCase {
	private static boolean isInitialised = false;
	private static IPackage model;
	
	public void setUp() throws Exception{
			if (!isInitialised) {
				Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put("xsd",
						new XSDResourceFactoryImpl());
				isInitialised = true;
			}
			XSDModelLoader loader = new XSDModelLoader();
			model = loader.load(new File("testdata/schema/xsdModelUtilsSchema.xsd"));
	}
	
	public void testGetAtributeByNameForElementWithNonModelFriendlyName() throws Exception {
		IClassifier typeOne = (IClassifier)model.getElementByName("TypeOne", true);
		
		IAttribute match = ModelUtils.getAttributeByOriginalName(typeOne, "element.One", true);
		assertEquals("element.One", match.getOriginalName());
	}
	
	public void testGetAttributeByNameForElementWithModelFriendlyName() throws Exception {
		IClassifier typeOne = (IClassifier)model.getElementByName("TypeOne", true);
		
		IAttribute match = ModelUtils.getAttributeByOriginalName(typeOne, "elementTwo", true);
		assertEquals("elementTwo", match.getOriginalName());
	}
	
	public void testGetAttributeByNameForAttributeWithModelFriendlyName() throws Exception {
		IClassifier typeOne = (IClassifier)model.getElementByName("TypeOne", true);
		
		IAttribute match = ModelUtils.getAttributeByOriginalName(typeOne, "attributeOne", true);
		assertEquals("attributeOne", match.getOriginalName());
	}
	
	public void testGetAttributeByNameForAttributeWithNonModelFriendlyName() throws Exception {
		IClassifier typeOne = (IClassifier)model.getElementByName("TypeOne", true);

		IAttribute match = ModelUtils.getAttributeByOriginalName(typeOne, "attribute.Two", true);
		assertEquals("attribute.Two", match.getOriginalName());
	}

	public void testGetAttributeByNameReturnsNullForNonExistentAttribute() throws Exception {
		IClassifier typeOne = (IClassifier)model.getElementByName("TypeOne", true);
		
		IAttribute match = ModelUtils.getAttributeByOriginalName(typeOne, "meh", true);
		assertNull(match);
	}
}
